/// <reference path="./eServiceFormHelpers.js"/>

var holidayApi_QueryBusinessDays = null;

/**
 * Configures the Booking Form with all the required validation and Views
 * @param {{}} options Optional Parameters to control the Loading of the Form
 * @returns {Object.<string, InitializeLogic>} The Booking Initialize Logic
 */
function CreateReferralRequestInitializationLogic(options) {
   if (options == null || options == undefined) {
      options = {};
   }

   let initalizeLogic = {};

   //#region Begin Initialization
   createInitializeLogic(initalizeLogic, "esdc_browseragent", function () {
      SaveUserAgent("esdc_browseragent");
      $("#esdc_browseragent").attr("readonly", "readonly");
   });

   createInitializeLogic(initalizeLogic, "esdc_languageofservice", function () {
      $("#esdc_languageofservice").change(function () {
         let esdc_languageofservice = $(this).val();
         if (esdc_languageofservice == 564190002) {
            $("#esdc_otherlanguage").parents("td").show();
            addRequiredValidator("esdc_otherlanguage", null, null);
            configureRequiredLabels({
               fields: ["esdc_otherlanguage"]
            });
         } else {
            $("#esdc_otherlanguage").parents("td").hide();
            removeRequiredValidator("esdc_otherlanguage");
         }
      });
      $("#esdc_languageofservice").change();
   });
   createInitializeLogic(initalizeLogic, "esdc_otherlanguage", function () {
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "emailaddress", function () {
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "esdc_usertimezone", function () {
      try {
         var timezone = Intl.DateTimeFormat().resolvedOptions().timeZone;
         $("#esdc_usertimezone").val(timezone);
      } catch (ex) { }
   });

   createInitializeLogic(initalizeLogic, "esdc_lastname", function () {
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "esdc_preferredname", function () {
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "esdc_preferredpronoun", function () {
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "esdc_provinceterritoryid", async function () {
      // Set the Loading text before we load the data
      let loadingText = getTranslatedText("Loading...", "Chargement en cours...");
      let defaultOption = $("#esdc_provinceterritoryid").children().first();
      let defaultOptionOriginalLabel = defaultOption.attr("label");
      let defaultOptionOriginalAria = defaultOption.attr("aria-label");

      defaultOption.attr("label", loadingText);
      defaultOption.attr("aria-label", loadingText);

      // Retrieve and Parse the Address data 
      let addressMetadata = await loadAddressMetadata(null);
      let provinces = addressMetadata.provinceTerritoryStates.value.filter(x => {
         /* Search for Canadian Provinces Canada */
         if (x._esdc_countryid_value == "0cf5389e-97ae-eb11-8236-000d3af4bfc3") {
            return true;
         }
         return false;
      }).map(x => {
         return {
            id: x.esdc_provinceterritorystateid,
            english: x.esdc_nameenglish,
            french: x.esdc_namefrench,
            text: getTranslatedText(x.esdc_nameenglish, x.esdc_namefrench),
            value: x.esdc_provinceterritorystateid,
            groupId: x.esdc_groupkey,
            record: x,
            attributes: [{
               key: "data-name-english",
               value: x.esdc_nameenglish
            },
            {
               key: "data-name-french",
               value: x.esdc_namefrench
            },
            {
               key: "data-countryid",
               value: x._esdc_countryid_value
            }
            ]
         }
      });

      provinces.push({
         id: "",
         english: "Select your Province/Territory",
         french: "Sélectionnez votre province/territoire",
         text: getTranslatedText("Select your Province/Territory", "Sélectionnez votre province/territoire"),
         value: ""
      });

      translateLookupNameField("esdc_provinceterritoryid", {
         idField: "esdc_provinceterritoryid",
         nameField: "esdc_provinceterritoryid option",
         list: provinces,
         noMatchFoundMode: "remove"
      });

      // Remove the Loading Text
      defaultOption.attr("label", defaultOptionOriginalLabel);
      defaultOption.attr("aria-label", defaultOptionOriginalAria);

      // Configure the Minimum Validator
      removeRequiredValidator("esdc_provinceterritoryid");
      createValidator("esdc_provinceterritoryid", null, "LookupFieldSelection", "esdc_provinceterritoryid", true, getTranslatedText(
         "is a required field.",
         "est un champ obligatoire."
      ), function () {
         let esdc_locationcountryid_value = $("#esdc_locationcountryid").val();
         if (esdc_locationcountryid_value == "d09b7e05-0952-ec11-8f8e-000d3a0a0f99") /* Outside of Canada */ {
            setLookupField("esdc_provinceterritoryid", null);
            return true;
         }

         let selectedValue = $("#esdc_provinceterritoryid").val();
         if (isNullOrWhitespace(selectedValue) || selectedValue == "-1") {
            return false;
         }
         return true;
      });


      // undo #766463 SDP Info - Other Organization 
      // $("#esdc_provinceterritoryid").change(
      //    function () {
      //       let pval = $("#esdc_provinceterritoryid").val();
      //       if (isNullOrWhitespace(pval) || pval == "-1") {
      //          $('[data-name=step_1_tab_section_9]').parent().hide();
      //       }
      //       else {
      //          $('[data-name=step_1_tab_section_9]').parent().show();
      //       }
      //    }
      // );
      // $("#esdc_provinceterritoryid").change();

      $("#esdc_provinceterritoryid").change(function () {

         let selectedValue = $("#esdc_provinceterritoryid").val();

         if (isNullOrWhitespace(selectedValue) || selectedValue == "-1") {
            $('[data-name=step_1_tab_section_9]').parent().hide();
            removeRequiredValidator("esdc_servicedeliverypartnerid");
         } else {
            $('[data-name=step_1_tab_section_9]').parent().show();
            addRequiredValidator("esdc_servicedeliverypartnerid", null, null);
         }
         serviceOfferingsLookupModalAccessibilityInfo();

         let callbackEnable = $("#esdc_callbacktimeenable").val();
         if (callbackEnable == "0" && !isNullOrWhitespace(selectedValue)) {
            loadSelectTimeSection(provinceValue);
         } else {
            loadSelectTimeSection([]);
            return;
         }
      });
      $("#esdc_provinceterritoryid").change();
   });

   createInitializeLogic(initalizeLogic, "esdc_servicedeliverypartnerid", function () {
      var englishSdpBodyHeader = getTranslatedText("Choose one record and click Select to continue", "Choisissez un enregistrement et cliquer Sélectionner pour continuer");
      let reqSpanElement = $("#esdc_servicedeliverypartnerid_lookupmodal .entity-lookup .modal-content .entity-grid .view-grid").find("span").eq(0);
      $(reqSpanElement).text(englishSdpBodyHeader);
      //ADO#832863 Platform - Remove tabindex from Lookup Modal
      $(reqSpanElement).removeAttr("tabindex");
   });



   // undo #766463 SDP Info - Other Organization 
   // createInitializeLogic(initalizeLogic, "esdc_isotherorganization", function () {
   //    $("#esdc_isotherorganization").change(function () {
   //       let esdc_isotherorganization = $(this).val();

   //       if (esdc_isotherorganization == 0) {
   //          $("#esdc_otherorganization").parents("td").show();
   //          addRequiredValidator("esdc_otherorganization", null, null);
   //          configureRequiredLabels({
   //             fields: ["esdc_otherorganization"]
   //          });
   //          removeRequiredValidator("esdc_servicedeliverypartnerid");
   //       }
   //       else {
   //          $("#esdc_otherorganization").parents("td").hide();
   //          removeRequiredValidator("esdc_otherorganization");
   //          addRequiredValidator("esdc_servicedeliverypartnerid", null, null);
   //          configureRequiredLabels({
   //             fields: ["esdc_servicedeliverypartnerid"],
   //             hide: [true]
   //          });
   //       }
   //    });
   //    $("#esdc_isotherorganization").change();
   // });

   // ADO#832838 Remove additional aria-label and title attributes  accessibility fix
   createInitializeLogic(initalizeLogic, "esdc_clienttelephonenumber", function () {
      $(this.jquery).removeAttr("aria-label");
      $(this.jquery).removeAttr("title");
   });

   createInitializeLogic(initalizeLogic, "esdc_callbacktimeenabled", function () {

      $("#esdc_callbacktimeenabled").change(
         function () {
            let provinceValue = $("#esdc_provinceterritoryid").val();
            let enableValue = $("#esdc_callbacktimeenabled").val();

            if (enableValue == "0" && !isNullOrWhitespace(provinceValue)) {
               loadSelectTimeSection(provinceValue);
            } else {
               loadSelectTimeSection([]);
               return;
            }
         }
      );
      $("#esdc_callbacktimeenabled").change();
   });


   createInitializeLogic(initalizeLogic, "step1_tab_section_selecttime_fieldset", function () {

      let tdColumn = this.jquery.find("tr").find("td").first();

      // Create the Selectors
      // WET Calendar - https://wet-boew.github.io/v4.0-ci/demos/cal-events/cal-events-en.html
      let availabilitySelector = $(document.createElement("div"));
      availabilitySelector.attr("id", "selectTime");
      availabilitySelector.addClass("row");
      //availabilitySelector.hide();
      tdColumn.append(availabilitySelector);

      let dateSelectorContainer = $(document.createElement("div"));
      dateSelectorContainer.addClass("col-xs-12");
      dateSelectorContainer.addClass("col-md-6");
      availabilitySelector.append(dateSelectorContainer);

      let timeSelectorContainer = $(document.createElement("div"));
      timeSelectorContainer.addClass("col-xs-12");
      timeSelectorContainer.addClass("col-md-6");
      availabilitySelector.append(timeSelectorContainer);




      let provinceTerritoryId = $("#esdc_provinceterritoryid").val();
      //let provinceHolidays = HolidayApi_MakeApiCall("QueryBusinessDays", "esdc_provinceterritorystate", "DAF4D05B-37B3-EB11-8236-0022486D8D5F", {});



      let holidays = ["2023-08-02T00:00:00+00:00", "2023-08-03T00:00:00+00:00"];

      let businessdays = [];
      let today = moment().add(1, "days");
      for (i = 0; i < 40; i++) {

         for (y = 0; y < holidays.length; y++) {
            if (holidays[y] == today.year().toString() && holidays[y] == today.month() && holidays[y] == today.month()) {
               today.add(1, "days");
               continue;
            }
            if (today.day() == 0 || today.day() == 6) {

               today.add(1, "days");
               continue;
            }

            businessdays.push(today.toDate());
            today.add(1, "days");

         }


      }

      // Create Date Selector Components
      let dateSelector = createPortalField("selectTime_date", {
         labelText: getTranslatedText(
            "Date",
            "Date"
         ),

      }, true, {

         containerType: "div",
         elementType: "select",
         defaultValue: null,
         visible: true,
         readonly: false,
         options: createOptionList(businessdays.map(function (x) {
            return {
               value: (x.toISOString()).substring(0, (x.toISOString()).indexOf('T')),
               text: x.toLocaleDateString(getTranslatedText("en", "fr"), {
                  weekday: "long",
                  year: "numeric",
                  month: "long",
                  day: "numeric"
               }),
               groupId: null,
               attributes: []
            };
         }), null, []).map(function (x) {
            return {
               element: x
            };
         })

      });
      $(dateSelector).addClass("col-xs-12");
      $(dateSelector).attr("id", "selectTime_date_container");
      dateSelectorContainer.append(dateSelector);

      // Create the Input Group for Date
      // createInputGroup("selectTime_date", {
      //    element: dateSelector,
      //    parent: ".control",
      //    buttons: [
      //       {
      //          content: {
      //             glyphicon: {
      //                position: "right",
      //                type: "glyphicon-step-forward",
      //                aria: getTranslatedText("Next Available Time", "Prochaine plage horaire disponible"),
      //                role: "button"
      //             }
      //          },
      //          elementType: "button",
      //          buttonType: "btn-default",
      //          position: "right",
      //          role: "button",
      //          type: "button",
      //          attributes: [
      //             { key: "data-date-id", value: "selectTime_date" },
      //             { key: "data-time-id", value: "selectTime_time" },
      //             { key: "data-date-index", value: 0 },
      //             { key: "data-time-index", value: -1 },
      //             { key: "data-time-length", value: -1 }
      //          ],
      //          action: function () {
      //             console.log("Finding Next Available Timeslot");
      //             let dateIndex = parseInt($(this).attr("data-date-index"));
      //             let timeIndex = parseInt($(this).attr("data-time-index")) + 1;

      //             // Retrieve all the Date Options
      //             let dateOptions = $("#selectTime_date").find("option:not([value=-1])");
      //             let currentDate = dateOptions.eq(dateIndex);
      //             let currentDateValue = currentDate.attr("value");

      //             // Retrieve the Time Slots, wrapping to the next date if overflowed
      //             let timeOptions = groupTimeslotsForDate(currentDateValue);
      //             let timeOptionKeys = Object.keys(timeOptions);
      //             let timeOptionsCount = timeOptionKeys.length;
      //             $(this).attr("data-time-length", timeOptionsCount);

      //             // Wrap the Time and re-retrieve the time options
      //             if (timeIndex >= timeOptionsCount) {
      //                dateIndex++;
      //                if (dateIndex >= dateOptions.length) { dateIndex = 0; }

      //                timeIndex = 0;
      //                currentDate = dateOptions.eq(dateIndex);
      //                currentDateValue = currentDate.attr("value");
      //                timeOptions = groupTimeslotsForDate(currentDateValue);
      //                timeOptionKeys = Object.keys(timeOptions);
      //                timeOptionsCount = timeOptionKeys.length;
      //                $(this).attr("data-time-length", timeOptionsCount);
      //             }

      //             let currentTimeValue = timeOptionKeys[timeIndex];
      //             console.log(`Next Available Time: ${currentDateValue} @ ${currentTimeValue} - dateIndex:${dateIndex}, timeIndex:${timeIndex}`);

      //             $(this).attr("data-date-index", dateIndex);
      //             $(this).attr("data-time-index", timeIndex);
      //             $("#selectTime_date").val(currentDate.attr("value"));
      //             $("#selectTime_date").change();
      //             $("#selectTime_time").val(currentTimeValue);
      //             $("#selectTime_time").change();
      //          }
      //       }
      //    ]
      // });

      let calendarDateContainer = $(document.createElement("div"));
      calendarDateContainer.addClass("col-xs-12");
      calendarDateContainer.attr("id", "selectTime_date_calendar_container");
      calendarDateContainer.attr("aria-hidden", false);
      calendarDateContainer.attr("tabindex", "-1");
      dateSelectorContainer.append(calendarDateContainer);

      let calendarDateDiv = $(document.createElement("div"));
      calendarDateDiv.css("margin-top", "1em");
      calendarDateDiv.css("margin-bottom", "1em");
      calendarDateDiv.attr("aria-hidden", false);
      calendarDateDiv.attr("tabindex", "-1");
      calendarDateContainer.append(calendarDateDiv);

      // Create Time Selector Components
      let timeSelector = createPortalField("selectTime_time", {
         labelText: getTranslatedText(
            "Time",
            "Heure"
         )
      }, true, {
         containerType: "div",
         elementType: "select",
         defaultValue: "",
         visible: true,
         readonly: false,
         options: [{
            value: "T08:30:00",
            text: "8:30 AM (8:30)"
         },
         {
            value: "T09:00:00",
            text: "9:00 AM (9:00)"
         },
         {
            value: "T09:30:00",
            text: "9:30 AM (9:30)"
         },
         {
            value: "T10:00:00",
            text: "10:00 AM (10:00)"
         },
         {
            value: "T10:30:00",
            text: "10:30 AM (10:30)"
         },
         {
            value: "T11:00:00",
            text: "11:00 AM (11:00)"
         },
         {
            value: "T11:30:00",
            text: "11:30 AM (11:30)"
         },
         {
            value: "T12:00:00",
            text: "12:00 PM (12:00)"
         },
         {
            value: "T12:30:00",
            text: "12:30 PM (12:30)"
         },
         {
            value: "T13:00:00",
            text: "1:00 PM (13:00)"
         },
         {
            value: "T13:30:00",
            text: "1:30 PM (13:30)"
         },
         {
            value: "T14:00:00",
            text: "2:00 PM (14:00)"
         },
         {
            value: "T14:30:00",
            text: "2:30 PM (14:30)"
         },
         {
            value: "T15:00:00",
            text: "3:00 PM (15:00)"
         },
         {
            value: "T15:30:00",
            text: "3:30 PM (15:30)"
         },
         ],
         attributes: [{
            key: "data-backingid",
            value: "selectTime_time"
         }]

         //let z = Array.from({length: 16}, (_, i) => (((i + 16) >> 1) + ":" + ((i + 0)%2*3) + "0").replace(/^\d:/, "0$&"));
      });

      $(timeSelector).addClass("col-xs-12");
      $(timeSelector).attr("id", "selectTime_time_container");
      timeSelectorContainer.append(timeSelector);

      let calendarTimeContainer = $(document.createElement("div"));
      calendarTimeContainer.addClass("col-xs-12");
      calendarTimeContainer.attr("id", "selectTime_time_calendar_container");
      calendarTimeContainer.attr("aria-hidden", true);
      calendarTimeContainer.attr("tabindex", "-1");
      timeSelectorContainer.append(calendarTimeContainer);

      let calendarTimeDiv = $(document.createElement("div"));
      calendarTimeDiv.css("margin-top", "1em");
      calendarTimeDiv.css("margin-bottom", "1em");
      calendarTimeDiv.attr("aria-hidden", true);
      calendarTimeDiv.attr("tabindex", "-1");
      calendarTimeContainer.append(calendarTimeDiv);





      // Create a container for actions
      let actionsContainer = $(document.createElement("div"));
      actionsContainer.attr("id", "step1_tab_section_selecttime_fieldset_actions");
      actionsContainer.addClass("pull-right");
      $(actionsContainer).insertAfter($("#step1_tab_section_selecttime_fieldset").find('legend'));

      // Create a container for messaging
      let messagingContainer = $(document.createElement("div"));
      messagingContainer.attr("id", "step1_tab_section_selecttime_fieldset_messaging");
      $(messagingContainer).insertAfter($("#step1_tab_section_selecttime_fieldset").find('legend'));
   });

   createInitializeLogic(initalizeLogic, "esdc_callbacktime", function () {
      createValidator("esdc_callbacktime", null, "UpdateCallbackTime", "esdc_callbacktime", false, "", function () {
         try {
            let selectTime_Date = $("#selectTime_date").val();
            let selectTime_Time = $("#selectTime_time").val();
            if (!isNullOrWhitespace(selectTime_Time)) {
               setDateTimeField("esdc_callbacktime", moment(selectTime_Date.concat(selectTime_Time)).utc(), true, false);

               //For email
               let callbackDT = moment(selectTime_Date.concat(selectTime_Time));
               setDateTimeField("esdc_callbacktimeforemail", callbackDT, false, false);
            }
         } catch (ex) { }
         return true;
      });
   });

   createInitializeLogic(initalizeLogic, "esdc_ReferralRequestServiceOfferings", function () {
      const validErrorText = getTranslatedText(
         " Please select at least one of the checkboxes under 'How can we help you? Please select all that apply.'",
         " Veuillez cocher au moins une des cases sous « Comment pouvons-nous vous aider ? Veuillez sélectionner tout ceux qui s'appliquent. »"
      );
      createValidator("esdc_ReferralRequestServiceOfferings", "", "RequiredFieldValidator", "esdc_ReferralRequestServiceOfferings", true, validErrorText, function () {    
         let checkedCount = 0;
         $("table[data-name=step_1_serviceoffering_section]").find("input").each(function () {
            if ($(this).is(':checked')) {
               checkedCount++;
            }
         });
         return checkedCount>0;
      });
      //$("table[data-name=step_1_serviceoffering_section] input[type=checkbox]").change();
   });

   //#endregion

   return initalizeLogic;
}

/**
 * Unlocks the Next Button so that it can be selected
 */
function unlockNextButton() {
   console.log($(this).attr("id") + " Changed, unlocking button");
   $("#NextButton").removeAttr("disabled");
}

async function createReferralServiceOfferingSection() {
   let groupings = await loadMetadata("/_api/esdc_portallistgroupings?$filter=statecode eq 0", "esdc_portallistgroupings", null, null);
   //console.log("Query has returned" + questions.value);

   let categories = [];
   for (let y = 0; y < groupings.value.length; y++) {
      let group = groupings.value[y];
      if (group.esdc_displayonportalreferralrequest == true) {
         categories.push({
            group: group
         });
      }
   }

   //Sort the array based on the Portal Group index 
   categories.sort(function (a, b) {
      return a.group.esdc_index - b.group.esdc_index;
   });

   console.log("# Categories on Portal " + categories.length);

   let containerElement = $('table[data-name="step_1_serviceoffering_section"] tbody tr:first td:first');
   for (let y = 0; y < categories.length; y++) {
      let ServiceOffering = categories[y];

      // Render table
      let table = document.createElement("table");
      containerElement.append(table);

      let tbdy = document.createElement("tbody");
      table.appendChild(tbdy);

      let tr = document.createElement('tr');
      tbdy.appendChild(tr);

      let td = document.createElement('td');
      td.setAttribute("colspan", "1");
      td.setAttribute("rowspan", "1");
      $(td).attr("class", "clearfix cell checkbox-cell");
      tr.appendChild(td);

      // Create the tooltip metadata
      let fieldTooltips = [];

      // Configure the Top Tooltip
      if (ServiceOffering.group.esdc_toptooltipenable == true) {
         if (!isNullOrWhitespace(ServiceOffering.group.esdc_toptooltipenglish) || !isNullOrWhitespace(ServiceOffering.group.esdc_toptooltipfrench)) {
            let tooltipTop = {
               id: "PortalListGrouping" + y + "_tooltip_top",
               elementType: "div",
               position: "above",
               innerHtml: getTranslatedText(ServiceOffering.group.esdc_toptooltipenglish, ServiceOffering.group.esdc_toptooltipfrench),
               isTooltip: true,
               descriptionType: "tooltip"
            };
            fieldTooltips.push(tooltipTop);
         }
      }

      // Configure the Bottom Tooltip
      if (ServiceOffering.group.esdc_bottomtooltipenable == true) {
         if (!isNullOrWhitespace(ServiceOffering.group.esdc_bottomtooltipenglish) || !isNullOrWhitespace(ServiceOffering.group.esdc_bottomtooltipfrench)) {
            let tooltipBottom = {
               id: "PortalListGrouping" + y + "_tooltip_bottom",
               elementType: "div",
               position: "below",
               innerHtml: getTranslatedText(ServiceOffering.group.esdc_bottomtooltipenglish, ServiceOffering.group.esdc_bottomtooltipfrench),
               isTooltip: true,
               descriptionType: "tooltip"
            };
            fieldTooltips.push(tooltipBottom);
         }

      }


      // Create the Service Offerings to pick
      let referralOfferings = createPortalField("referralOffering" + y, {
         labelText: getTranslatedText(ServiceOffering.group.esdc_nameenglish, ServiceOffering.group.esdc_namefrench)
      }, false, {
         elementType: "input",
         attributes: [{
            key: "class",
            value: ""
         },
         {
            key: "type",
            value: "checkbox"
         },
         {
            key: "data-categoryid",
            value: ServiceOffering.group.esdc_portallistgroupingid
         },
         {
            key: "data-referralrequest-checkbox-n2n",
            value: true
         }
         ],
         description: fieldTooltips
      });
      td.appendChild(referralOfferings);

   }
}

async function createReferralRequestMappingsAPI() {
   console.log("createReferralRequestMappingsAPI")

   //Obtain the record information to associate to the question
   let recordInfo = getRecordInformation();

   // Log all the requests
   let allAPIRequestPromises = [];
   let allAPIRequestData = [];

   // Old Case Intake Error
   // let error;
   // if (isEnglish) { error = 'Please select at least one of the checkboxes under "How can we help you?"'; }
   // else { error = "Veuillez cocher au moins une des cases sous « Comment pouvons-nous vous aider? »"; }

   // Configure the validation on the Service Offering selections
   // createValidator("esdc_ReferralRequestServiceOfferings", "", "", null, false, getTranslatedText(
   //    " Please select at least one of the checkboxes under 'How can we help you?'",
   //    " Veuillez cocher au moins une des cases sous « Comment pouvons-nous vous aider? »"
   // ), function () {
   //    let offeringChecked = $("input[type=checkbox]:checked");
   //    if (offeringChecked.length == 0) {
   //       return false;
   //    }
   //    return true;
   // });

   //Obtain all the questions that have been checked
   let offeringChecked = $("input[type=checkbox]:checked");
   for (let i = 0; i < offeringChecked.length; i++) {
      let offering = offeringChecked[i];

      // Get te Offering Id
      let offID = $(offering).attr("data-categoryid");
      if (isNullOrWhitespace(offID)) {
         console.log(`offID: ${offID} is invalid. Skipping association`);
         continue;
      }
      let apiData = {
         "@odata.id": window.location.origin + "/_api/esdc_portallistgroupings({0})".replace("{0}", offID),

      };
      allAPIRequestData.push(apiData);

      try {
         console.log("Attempting to Associate: esdc_portallistgroupings with Referral Request: " + JSON.stringify(apiData));
         let requestPromise = safeAjax({
            type: "POST",
            url: "/_api/esdc_referralrequests({0})/esdc_referralrequest_esdc_portallistgroup/$ref".replace("{0}", recordInfo.id),
            contentType: "application/json",
            data: JSON.stringify(apiData),
            success: function (res, status, xhr) {
               let recordId = xhr.getResponseHeader("entityid");
               console.log("Record Created: esdc_referralrequest" + recordId);
            },
            error: function (XMLHttpRequest, textStatus, errorThrown) {
               console.log("There was an error");
               console.log(errorThrown);
            }
         });
         console.log("Request Promise: " + requestPromise);
         allAPIRequestPromises.push(requestPromise);
      } catch (ex) {
         console.log("An exception occured creating Mapping record");
         console.log(ex);
      }
   }

   // Log all the API Requests to the JSON field
   $("#esdc_questionmappingjson").val(JSON.stringify(allAPIRequestData));

   // Await all of the promises to be completed
   return await Promise.allSettled(allAPIRequestPromises);
}

/**
 * Preforms an API Call to gather the available timeslots or book an appointment.
 * @param {"QueryBusinessDays"} mode The mode of the API Call
 * @param {string} logicalname The name of the entity
 * @param {string} recordId The Id of the calendar, province or office
 * @param {{
 *    apiUrl: string?,
 *    onSuccess: Function,
 *    onError: Function,
 *    payload: {
 *    }?
 * }} options Additional Options for the API Call
 */
async function HolidayApi_MakeApiCall(mode, logicalname, recordId, options) {
   if (isNullOrWhitespace(options.apiUrl)) {
      options.apiUrl = $("#eServiceAPIUrl").val();
   }

   // Calculate the CrmOrigin
   let environmentOrigin = window.top.location.host.substring(0, window.top.location.host.indexOf("."));

   // Create the Api Payload
   let fullApiUrl = options.apiUrl + "api/" + mode;
   let apiPayload = {
      // "Mode": mode,
      "CrmOrigin": environmentOrigin,
      "Record": {
         "LogicalName": logicalname,
         "Id": recordId
      }
   };

   // Send the Request
   return new Promise((resolve, reject) => {
      let ajaxApiDefinition = {
         method: "POST",
         headers: {
            "CrmOrigin": environmentOrigin
         },
         data: JSON.stringify(apiPayload),
         contentType: 'application/json; charset=utf-8',
         dataType: 'json',
         url: fullApiUrl,
         success: function (data, status, xhr) {
            console.log("Success: HolidayApi_MakeApiCall");
            let dataJson = data;
            data = convertJSONToObject(data);

            // console.log(data);
            // console.log(status);
            // console.log(xhr);

            if (variableExists(options.onSuccess)) {
               options.onSuccess(data, status, xhr);
            }

            resolve({
               data: data,
               dataJson: dataJson,
               status: status,
               xhr: xhr
            });
         },
         error: function (jqXhr, textStatus, errorMessage) {
            console.log(`Error: HolidayApi_MakeApiCall - ${textStatus}: ${errorMessage}`);
            if (variableExists(options.onError)) {
               options.onError(jqXhr, textStatus, errorMessage);
            }

            reject({
               jqXhr: jqXhr,
               textStatus: textStatus,
               errorMessage: errorMessage
            });
         }
      };

      $.ajax(ajaxApiDefinition);
   });
}


/**
 * Loads in the Selected Time Sections
 * @param {string} provinceId The selected province to use for loading the Select Time section 
 * @returns 
 */
async function loadSelectTimeSection(provinceId) {
   let selectTimeSection = $("#step1_tab_section_selecttime_fieldset");

   // If we dont have an Office Id selected, then hide the section and exit
   if (provinceId.length == 0) {
      selectTimeSection.hide();
      removeRequiredValidator("selectTime_date");
      removeRequiredValidator("selectTime_time");
      return;
   }


   // Create Validators for the Virtual Fields
   addRequiredValidator("selectTime_date");
   addRequiredValidator("selectTime_time");
   configureRequiredLabels({
      fields: ["selectTime_date", "selectTime_time"]
   });

   // Show the Select Time Section, and animate a scroll to it
   selectTimeSection.show();
   animateScrollToElement(selectTimeSection, 400);

   // Begin loading the availabilities
   HolidayApi_MakeApiCall(
      "QueryBusinessDays",
      "esdc_provinceterritorystate",
      provinceId, {
      onSuccess: function (data, status, xhr) {
         if (!data) {
            console.log("error fetching data");
            return;
         }

         let today = moment().startOf("day");
         let holidays = data.Holidays.map(x => {
            let date = moment.utc(x);
            return {
               raw: x,
               key: date.format("YYYY-MM-DD"),
               momentDate: date,
               unixTime: date.unix()
            };
         })
            .filter(x => {
               let today = moment();
               return x.unixTime >= today.unix()
            })
            .sort((a, b) => (a.unixTime > b.unixTime) ? 1 : ((b.unixTime > a.unixTime) ? -1 : 0));

         const numberDaysFuture = 40;
         let businessdays = [];
         for (i = 0; i < numberDaysFuture; i++) {
            today.add(1, "days");
            // Skip Weekends
            if (today.day() == 0 || today.day() == 6) {
               continue;
            }

            // Check Holidays
            let isHoliday = false;
            for (let iHoliday = 0; iHoliday < holidays.length; iHoliday++) {
               let holidayTmp = holidays[iHoliday];
               //Check the year, month and date on the holiday to the incremented days we compare with
               if ((holidayTmp.momentDate.year() == today.year()) && (holidayTmp.momentDate.month() == today.month()) && (holidayTmp.momentDate.date() == today.date())) {
                  isHoliday = true;
                  break;
               }
            }

            if (isHoliday) {
               continue;
            }

            businessdays.push(today.toDate());
         }

         let dateOptions = createOptionList(businessdays.map(function (x) {
            return {
               value: x.toISOString().substring(0, x.toISOString().indexOf('T')),
               text: x.toLocaleDateString(getTranslatedText("en", "fr"), {
                  weekday: "long",
                  year: "numeric",
                  month: "long",
                  day: "numeric"
               }),
               groupId: null,
               attributes: []
            };
         }), null, []);

         // Clear out the old selectors and add the new ones
         let selectTime_Date = $("#selectTime_date");
         selectTime_Date.children().remove();
         selectTime_Date.append(dateOptions);

         $("#selectTime_date").val("");

         console.log("Business Days", businessdays, selectTime_Date, dateOptions);
         console.log("Date Selector", selectTime_Date, dateOptions);
      },
      onError: function (jqXhr, textStatus, errorMessage) {
         $("#selectTime").hide();
         console.log("error on API request for holidays");
      }
   }
   )
}


/**
 * Used on the Confirmation Page. Creates an Add to Calendar Button for this Referral Request
 */
function createAddToCalendarButton() {
   loadStyle("https://cdn.jsdelivr.net/npm/add-to-calendar-button@1.8/assets/css/atcb.min.css");
   $.getScript("https://cdn.jsdelivr.net/npm/add-to-calendar-button@1.8", function (data, textStatus, jqxhr) {
      console.log("Creating 'Add to Calendar' Button");
      let confirmationNumber = $("#esdc_referralrequestnumber").val();
      let startDateTime = $("#esdc_callbacktime").val();
      let calendarItemName = getTranslatedText("eServiceCanada Referral Request - Scheduled Call", "[FR]eServiceCanada Referral Request - Scheduled Call")


      // https://jekuer.github.io/add-to-calendar-button/
      let addToCalendarBtn = $(document.createElement("div"))
         .addClass("atcb")
         .css("display", "none")
         .html(JSON.stringify(
            {
               "name": `#${calendarItemName} #${confirmationNumber}`,
               "description": `
                  ${calendarItemName} <br>
                  ${getTranslatedText("Referral Request Number", "[FR]Referral Request Number")}: ${confirmationNumber} <br>
                  ${getTranslatedText("Call Local Time", "[FR]Call Local Time")}: ${moment(startDateTime).format("YYYY-MM-DD")} @ ${moment(startDateTime).format("HH:mm")} <br>
               `,
               "timeZone": "currentBrowser",
               "startDate": moment(startDateTime).format("YYYY-MM-DD"),
               "endDate": moment(startDateTime).format("YYYY-MM-DD"),
               "startTime": moment(startDateTime).format("HH:mm"),
               "endTime": moment(startDateTime).format("HH:mm"),
               "label": getTranslatedText("Add Request Details to Calendar", "Ajouter les détails de la demande au calendrier"),
               "options": [
                  "Apple",
                  "Google",
                  "iCal",
                  "Microsoft365",
                  "MicrosoftTeams",
                  "Outlook.com",
                  "Yahoo"
               ],
               "iCalFileName": "eServiceCanada-Referral"
            }
         )).appendTo($("#esdc_callbacktime").parents("td"));
      atcb_init();
      $("#atcb_button_0").attr("type", "button");
   });
}


function enableDisableNextButtonOnTouPrivacyCheck() {
   if($('#touAndPrivacyCheck').is(':checked')) {
      $("#NextButton").removeAttr("disabled");
      console.log("Enable Next Button, agree checkBox checked");
   }
   else {
      $("#NextButton").attr("disabled", true);
      console.log("Disable Next Button, agree checkBox unChecked");
   }
}

//ADO# 832802 accessibility fix inserting required h2 element
function insertH2ElementInHeaderForAccessibilityFix() {
    let h2Element = document.createElement('h2');
    h2Element.setAttribute("class","portal-page-title");
    h2Element.setAttribute("dir","ltr");
    h2Element.textContent = getTranslatedText("Reaching All Canadians - Service Referral Initiative", "Servir tous les Canadiens - Initiative d'aiguillage vers les services");
    $("div .page-header").append(h2Element);
}


// ADO #832832, #832865 sort order related accesibility changes for esdc_servicedeliverypartnerid_lookupmodal
function serviceOfferingsLookupModalAccessibilityInfo() {
   let myUniqSpid = "sortTextSpanG";
   let sortInfoText = getTranslatedText("Click to Sort", "Cliquez pour trier");
   $("#esdc_servicedeliverypartnerid_lookupmodal table[role=grid] thead").find("th[scope=col].sort-enabled").each(function () {
      if ($(this).hasClass("sort") && $(this).hasClass("sort-asc")) {
         sortInfoText = getTranslatedText("Ascending Order", "Ordre croissant");
      }
      else if ($(this).hasClass("sort") && $(this).hasClass("sort-desc")) {
         sortInfoText = getTranslatedText("Descending Order", "Ordre décroissant");
      }
      let anchElement = $(this).find("a[role=button]");
      let spanElement = $(anchElement).find(`span[id=${myUniqSpid}]`);
      if (spanElement == undefined || spanElement.length == 0)
         $(anchElement).append(`<span id='${myUniqSpid}'> (${sortInfoText})<span>`);
      else
         $(spanElement).text(`(${sortInfoText})`);

      if ($(anchElement).attr("onclick")==undefined)
         $(anchElement).attr("onclick", "javascript:serviceOfferingsLookupModalAccessibilityInfo()");
   });
}